@@ -34,25 +34,40 @@ my $repeats = $conf->{'repeats'} || 10;
 while ( $combination = $engine->next() ) {
   my $secret_code = join("",@$combination);
   for ( 1..$repeats ) {
+    print "Code $secret_code\n";
     my $solver;
     eval "\$solver = new Algorithm::MasterMind::$method \$method_options";
     die "Can't instantiate $solver: $@\n" if !$solver;
     my $game = { code => $secret_code,
 		 combinations => []};
-    my $first_string = $solver->issue_first;
-    my $response =  check_combination( $secret_code, $first_string);
-    push @{$game->{'combinations'}}, [$first_string,$response] ;
+    my $move = $solver->issue_first;
+    my $response =  check_combination( $secret_code, $move);
+    push @{$game->{'combinations'}}, [$move,$response] ;
     
-    $solver->feedback( $response );
-    print "Code $secret_code\n";
-    my $played_string = $solver->issue_next;
-    while ( $played_string ne $secret_code ) {
-      print "Playing $played_string\n";
-      $response = check_combination( $secret_code, $played_string);
-      push @{$game->{'combinations'}}, [$played_string, $response] ;
+    while ( $move ne $secret_code ) {
       $solver->feedback( $response );
-      $played_string = $solver->issue_next;      
-    }  
+      $move = $solver->issue_next;
+      print "Playing $move\n";
+      $response = check_combination( $secret_code, $move);
+      push @{$game->{'combinations'}}, [$move, $response] ;
+      $solver->feedback( $response );
+      if ( $solver->{'_consistent'} ) {
+	push @{$game->{'consistent_set'}}, [ keys %{$solver->{'_consistent'}} ] ;
+      }  else {
+	my $partitions = $solver->{'_partitions'};
+	push @{$game->{'consistent_set'}}, 
+	  [ map( $_->{'_string'}, @{$partitions->{'_combinations'}}) ];
+	if ( $partitions->{'_score'}->{'_most'} ) {
+	  push @{$game->{'top_scorers'}}, [ $partitions->top_scorers('most') ];
+	} elsif ( $partitions->{'_score'}->{'_entropy'} ) {
+	  push @{$game->{'top_scorers'}}, [ $partitions->top_scorers('entropy') ];
+	}
+      }
+      if ( $solver->{'_data'} ) {
+	push @{$game->{'data'}}, $solver->{'_data'};
+      }
+    }
+
     $game->{'evaluations'} = $solver->evaluated();
     $io->print($game);
     print "Finished\n";
